;------------------------------------------------------------------------------
; serial.asm
;
; This module provides routines to output data to the serial port using the
; hardware asynchronous interface. It also supports printing strings received
; from the uM-FPU.
;
; The list and #include directives should be changed if another PIC device is
; used that has compatible features.
;
; The baud rate setting depends on the clock speed used (refer to the appropriate
; PIC datasheet). Sample settings are as follows:
;
;							  BRGH	  SPBRG
;							  ----	  -----					
; 		20 MHz	19,200 baud		1		64
; 		4 MHz	19,200 baud		1		12
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 September 20, 2004
; changes:
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list      p=16f628				;list directive to define processor
	#include <p16f628.inc>			;processor specific variable definitions

	#include "umfpu.inc"		
	extern	stringTable

;-------------------- local variables -----------------------------------------
	udata

#define RX_DIR		TRISB, 7			;RX direction bit
#define TX_DIR		TRISB, 6			;TX direction bit

sptr	res	1							;string pointer low 8 bits
tmp		res	1							;temporary value

PROG	code

;-------------------- print_setup ---------------------------------------------
; Initialize the serial port (defaults to 19,200 baud 8,N,1
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_setup
print_setup
	banksel		TRISB
	bsf			RX_DIR					;configure RX as input
	bcf			TX_DIR					;configure TX as output
	banksel		PORTB

	movlw		.64						;set baud rate to 19,200 (assumes 20 MHz)
	banksel		SPBRG
	movwf		SPBRG
	movlw		0x26					;set TXEN, BRGH, TRMT in TXSTA
	movwf		TXSTA
	movlw		0x90					;set SPEN, CREN in RCSTA
	banksel		RCSTA
	movwf		RCSTA
	return

;-------------------- print_version -------------------------------------------
; Print the uM-FPU version string
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_version
print_version
	movlw		XOP						;get version string from uM-FPU
	call		fpu_sendByte
	movlw		VERSION
	call		fpu_sendByte
	goto		print_fpuString			;print the string

;-------------------- print_float ---------------------------------------------
; Print floating point value
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_float, print_floatFormat
print_float
	clrw								;set format to zero
	;(fall through to print_floatFormat)

;-------------------- print_floatFormat ---------------------------------------
; Print formatted floating point value
;	input:	W register	format	
;	output:	none
;------------------------------------------------------------------------------

print_floatFormat
	movwf		tmp						;save the format value
	movlw		FTOA					;convert floating point to ASCII
	call		fpu_sendByte
	movf		tmp, w
	goto		print_format

;-------------------- print_long ----------------------------------------------
; Print long integer value
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_long, print_longFormat
print_long
	clrw								;set format to zero
	;(fall through to print_longFormat)

;-------------------- print_longFormat -----------------------------------------
; Print formatted long integer value
;	input:	W register	format	
;	output:	none
;------------------------------------------------------------------------------

print_longFormat
	movwf		tmp						;save the format value
	movlw		LTOA					;convert long integer to ASCII
	call		fpu_sendByte
	movf		tmp, w
	;(fall through to print_format)
		
print_format
	movf		tmp, w					;send the format byte
	call		fpu_sendByte
	;(fall through to print_fpuString)

print_fpuString
	call		fpu_wait				;wait until string is ready
	movlw		READSTR					;read string
	call		fpu_sendByte
	call		fpu_readDelay

print_fpuString2						;read zero terminated string
	call	fpu_readByte				;read the next byte
	btfsc	STATUS, Z					;check for zero terminator
	return								;yes, then exit

	call	print_byte					;no, then send the byte

	andlw	0x80						;check for character > 127
	btfsc	STATUS, Z
	goto	print_fpuString2			;yes, then get next byte
	return								;exit
	 	
;-------------------- print_string ---------------------------------------------
; Send zero terminated string to serial port
;	input:	W register	low byte of string address
;	output:	none
;------------------------------------------------------------------------------

	global	print_string
print_string
	movwf	sptr						;save the string offset
	decf	sptr, f						;adjust for computed goto

print_string2
	movlw	HIGH stringTable			;get high byte of string address (in case
	movwf	PCLATH						; stringTable stored on different page)

	movf	sptr, w						;get low byte of string address
	call	stringTable					;get the next character

	clrf	PCLATH						;reset PCLATH to page zero (in case
										; stringTable stored on different page)
	iorlw	0
	btfsc	STATUS, Z					;check for zero
	return								;yes, then exit

	call	print_byte					;send character to serial port

	incf	sptr, f						;increment string pointer
	goto	print_string2				;get next character

;-------------------- print_hex -----------------------------------------------
; Send 8-bit hex value to serial port
;	input:	W register	8-bit hex value
;	output:	none
;------------------------------------------------------------------------------

	global	print_hex
print_hex
	movwf	tmp							;save the original value
	swapf	tmp, w						;display upper hex digit
	call	print_hexDigit
	movf	tmp, w						;display lower hex digit			
	;(fall through to print_hexDigit)

;-------------------- print_hexDigit ------------------------------------------
; Send 4-bit hex value to serial port
;	input:	W register	lower 4 bits contain hex value
;	output:	none
;------------------------------------------------------------------------------

	global	print_hexDigit
print_hexDigit
	andlw	0x0F						;get the lower 4 bits
	addlw	6							;check for A-F digit
	btfsc	STATUS, DC
	addlw	7							;yes, then adjust
	addlw	'0'-6						;convert to ASCII character
	goto	print_byte					;send hex digit

;-------------------- print_crlf ----------------------------------------------
; Send carraige return, linefeed to serial port
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_crlf
print_crlf
	movlw	0x0D						;send carraige return
	call	print_byte
	movlw	0x0A						;send linefeed
	;(fall through to printByte)

;-------------------- print_byte -----------------------------------------------
; Send byte to serial port
;	input:	W register	8-bit byte
;	output:	none (original W register retained)
;------------------------------------------------------------------------------

	global	print_byte
print_byte
	btfss	PIR1, TXIF					;wait for buffer empty
	goto	print_byte
	movwf	TXREG						;send byte to serial port
	return
	end
