;------------------------------------------------------------------------------
;
; File: demo2.asm
;
; This program demonstrates how to use the uM-FPU floating point coprocessor
; connected to PIC microcontroller over a 3-wire SPI interface. It takes
; temperature readings from a DS1620 digital thermometer, converts them to
; floating point and displays them in degrees Celsius and degrees Fahrenheit.
; It also calculates the minimum and maxomum temperatures.
;
; The list, #include, and CONFIG settings should be changed if another PIC
; device with compatible features is used.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2005. All rights reserved.
;
; @version	V2.0 January 10, 2005
; changes:
;	V2.0 January 10, 2005
;	  - original code
;------------------------------------------------------------------------------

	list      p=16f877					;list directive to define processor
	#include <p16f877.inc>				;processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _HS_OSC & _WRT_ENABLE_ON & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 

	#include umfpu.inc					;uM-FPU definitions

	extern	delay_ms
	extern	print_setup, print_byte, print_crlf, print_string
	extern	print_version, print_floatFormat, print_hex

;-------------------- uM-FPU register definitions -----------------------------

#define		DegC			1			;degrees Celsius
#define		DegC_Min		2			;minimum temperature
#define		DegC_Max		3			;maximum temperature
#define		DegF			4			;degrees Fahrenheit

#define		F1_8			5			;constant 1.8
#define		F32				6			;constant 32.0

;-------------------- variable definitions ------------------------------------
	udata

#define		DS_RST		PORTA, 0		;DS1620 reset/enable
#define		DS_CLK		PORTA, 1		;DS1620 clock
#define		DS_DATA		PORTA, 2		;DS1620 data
#define 	DS_DATA_DIR	TRISA, 2		;direction bit

dataBits	res	1						;data bits
bitCount	res	1						;bit count
delayCnt	res	1						;counter for main loop delay
rawTemp		res	2						;raw temperature
tmp			res 1						;temporary value

;-------------------- string definitions --------------------------------------
; lower byte of stringTable address must be 00
; stringTable can have up to 255 bytes of data
; all strings must be zero terminated

STRINGS	code
	global	stringTable
stringTable
	addwf	PCL,f						;computed goto for strings

start1Message
	dt	0x0D, 0x0A
	dt	0x0D, 0x0A, "Demo2 - ", 0
start2Message
	dt	0x0D, 0x0A, "-------------------", 0x0D, 0x0A, 0
errorMessage
	dt	"uM-FPU not detected", 0
titleString
	dt	0x0D, 0x0A, "               Min   Current   Max"
	dt	0x0D, 0x0A, "              -----  -------  ----", 0
degCString
	dt 0x0D, 0x0A, "Degrees C:", 0
degFString
	dt 0x0D, 0x0A, "Degrees F:", 0
	
;-------------------- reset and interrupt vector ------------------------------

STARTUP	code
	nop								;reset vector
  	goto    reset
	nop
	nop
	goto	isr							;interrupt vector

;-------------------- interrupt service routine -------------------------------

PROG	code
isr
	retfie								;(no interrupts used)


;==============================================================================
;-------------------- initialization ------------------------------------------
;==============================================================================

reset		
 	call		print_setup				;initialize the serial port

	movlw		LOW start1Message		;display startup string
	call		print_string

	; reset the uM-FPU and display version
	;-------------------------------------

	call		fpu_reset				;reset the uM-FPU
	btfsc		STATUS, Z				;check status
	goto		reset2
	movlw		LOW errorMessage		;print error message if reset failed
	call		print_string
	goto		done

reset2
	call		print_version			;print uM-FPU version number
	movlw		LOW start2Message		;display underline
	call		print_string

	call		init_DS1620				;initizlize the DS1620

	; load constant 1.8
	;------------------
	movlw		F1_8					;select F1_8 as A register
	call		fpu_sendByte
	movlw		ATOF					;load the string 1.8 to the uM-FPU
	call		fpu_sendByte			; (note: string must be zero terminated)
	movlw		'1'						; convert to floating point,
	call		fpu_sendByte			; store value in register 0
	movlw		'.'						; select register 0 as the B register
	call		fpu_sendByte
	movlw		'8'
	call		fpu_sendByte
	movlw		0
	call		fpu_sendByte
	movlw		FSET					;F1_8 = register 0 (i.e. 1.8)
	call		fpu_sendByte

	; load constant 32.0
	;-------------------
	movlw		F32						;select F32 as A register
	call		fpu_sendByte
	movlw		LOADBYTE				;load the byte value 32 to register 0,
	call		fpu_sendByte			; convert to floating point,
	movlw		.32						; select register 0 as B register
	call		fpu_sendByte
	movlw		FSET					;F32 = register 0 (i.e. 32.0)
	call		fpu_sendByte

	movlw		LOW titleString			;display title
	call		print_string

	; read initial temperature
	;-------------------------	
	call		read_temperature		;read initial temperature

	;initialize min/max values
	;-------------------------
	movlw		DegC_Min				;select DegC_Min as A register
	call		fpu_sendByte
	movlw		FSET+DegC				;DegC_Min = DegC
	call		fpu_sendByte
	movlw		DegC_Max				;select DegC_Max as A register
	call		fpu_sendByte
	movlw		FSET+DegC				;DegC_Max = DegC
	call		fpu_sendByte
	
;==============================================================================
;-------------------- main routine --------------------------------------------
;==============================================================================

main
	;get temperature reading from DS1620
	;-----------------------------------
	call		read_temperature		;read temperature

	;calculate minimum/maximum temperature
	;-------------------------------------
	movlw		SELECTB+DegC			;select DegC as B register
	call		fpu_sendByte

	movlw		DegC_Min				;select Deg_Min as A register
	call		fpu_sendByte
	movlw		XOP						;Deg_Min = min of A and B
	call		fpu_sendByte
	movlw		FMIN
	call		fpu_sendByte

	movlw		DegC_Max				;select Deg_Max as A register
	call		fpu_sendByte
	movlw		XOP						;Deg_Max = max of A and B
	call		fpu_sendByte
	movlw		FMAX
	call		fpu_sendByte

	;display degrees Celsius
	;-----------------------
	movlw		LOW degCString			;display Celsius text string
	call		print_string

	movlw		DegC_Min				;display min/current/max
	call		print_tempC				;(in Celcius)
	movlw		DegC
	call		print_tempC
	movlw		DegC_Max
	call		print_tempC

	;display degrees Fahrenheit
	;--------------------------
	movlw		LOW degFString			;display Fahrenheit text string
	call		print_string

	movlw		DegC_Min				;display min/current/max
	call		print_tempF				;(in Fahrenheit)
	movlw		DegC
	call		print_tempF
	movlw		DegC_Max
	call		print_tempF
	call		print_crlf

	;delay for 2 seconds and repeat main loop
	;----------------------------------------
	movlw		8						;8 x 250 msec = 2 seconds
	movwf		delayCnt

pause
	movlw		.250					;delay for 1 second
	call		delay_ms
	decfsz		delayCnt, f				;loop for all bits
	goto		pause

	goto		main					;repeat main loop

;-------------------- print_tempC --------------------------------------------
; print the temperature in Celsius
; input: W register	- register number of temperature in Celsius
;-----------------------------------------------------------------------------

print_tempC
	call		fpu_sendByte			;select register passed as A register

	movlw		.81						;set format to 8,1
	call		print_floatFormat		;print floating point value
	return

;-------------------- print_tempF --------------------------------------------
; print the temperature in Fahrenheit
; input: W register	- register number of temperature in Celsius
;-----------------------------------------------------------------------------

print_tempF
	movwf		tmp						;save the register number

	;DegF = reg * 1.8 + 32
	;----------------------
	movlw		DegF					;select DegF as the A register
	call		fpu_sendByte
	movlw		FSET					;set DegF = value of register passed
	addwf		tmp, W
	call		fpu_sendByte
	movlw		FMUL+F1_8				;multiply DegF by 1.8
	call		fpu_sendByte
	movlw		FADD+F32				;add 32.0 to DegF
	call		fpu_sendByte

	movlw		.81						;set format to 8,1
	call		print_floatFormat		;print floating point value
	return

;-------------------- init_DS1620 --------------------------------------------
; initialize DS1620
;------------------------------------------------------------------------------

init_DS1620
	bcf			DS_RST					;initialize pins
	bsf			DS_CLK
	banksel		TRISA
	movlw		0x06					;congiure A0-A3 for digitial I/O
	movwf		ADCON1
	movlw		0xF8					;configure A0-A2 as outputs
	movwf		TRISA
	banksel		PORTA

	movlw		.100					;delay
	call		delay_ms

	bsf			DS_RST					;configure for CPU control
	movlw		0x0C					
	call		write_DS1620
	movlw		0x02
	call		write_DS1620
	bcf			DS_RST

	movlw		.100					;delay	
	call		delay_ms

	bsf			DS_RST					;start temperature conversions
	movlw		0xEE
	call		write_DS1620
	bcf			DS_RST

	movlw		.250					;delay 1 second
	call		delay_ms
	movlw		.250
	call		delay_ms
	movlw		.250
	call		delay_ms
	movlw		.250
	call		delay_ms
	return	

done
	goto		done					;error exit
	
;-------------------- write_DS1620 --------------------------------------------
; write DS1620 command
;------------------------------------------------------------------------------

write_DS1620
	movwf		dataBits				;save the byte to send
	movlw		.8						;get number of bits to send
	movwf		bitCount

write2
	bcf			DS_DATA					;set data output LOW
	rrf			dataBits, f				;get next data bit
	btfsc		STATUS, C				;if next bit is 1, set data output HIGH
	bsf			DS_DATA

	bcf			DS_CLK					;pulse the clock
	bsf			DS_CLK

	decfsz		bitCount, f				;loop for all bits
	goto		write2
	return

;-------------------- read_temperature ----------------------------------------
; read temperature value from DS1620 and store in rawTemp
;------------------------------------------------------------------------------

read_temperature
	bsf			DS_RST					;enable DS1620	

	movlw		0xAA					;send read temperature command
	call		write_DS1620

	banksel		TRISA
	bsf			DS_DATA_DIR				;configure DS_DATA as input
	banksel		PORTA

	movlw		.8						;get number of bits to receive
	movwf		bitCount

read2
	bcf			DS_CLK					;set clock LOW
	bcf			STATUS, C				;set carry to zero
	btfsc		DS_DATA					;check data input
	bsf			STATUS, C				;if HIGH, set carry to one
	bsf			DS_CLK					;set clock HIGH
	rrf			rawTemp, f				;store next data bit
	decfsz		bitCount, f				;loop for all bits
	goto		read2

	clrw								;clear high bits
	btfsc		DS_DATA					;check data input
	movlw		0xFF					;if HIGH, set high bits
	bsf			DS_CLK					;set clock HIGH
	movwf		rawTemp+1				;store high bits

	bsf			DS_CLK					;set clock HIGH
	banksel		TRISA
	bcf			DS_DATA_DIR				;configure DS_DATA as output
	banksel		PORTA
	bcf			DS_RST					;disable DS1620	

	;load rawTemp to uM-FPU, convert to floating point, and store in register
	;------------------------------------------------------------------------
	movlw		DegC					;select DegC as A register
	call		fpu_sendByte
	movlw		LOADWORD				;load rawTemp to register 0,
	call		fpu_sendByte			; convert to floating point,
	movf		rawTemp+1, w			; select register 0 as B register
	call		fpu_sendByte
	movf		rawTemp, w
	call		fpu_sendByte
	movlw		FSET					;degC = register 0 (i.e. set to the
	call		fpu_sendByte			; floating point value of rawTemp)

	;divide the raw value by 2 to get degrees Celsius
	;------------------------------------------------
	movlw		LOADBYTE				;load the value 2 to register 0,
	call		fpu_sendByte			; convert to floating point,
	movlw		.2						; select register 0 as B register
	call		fpu_sendByte
	movlw		FDIV					;divide DegC by register 0
	call		fpu_sendByte			; (i.e. divide by 2)

	return
	end


