;------------------------------------------------------------------------------
; fpusw_4.asm
;
; This module provides low-level interface routines for connecting 14-bit PIC
; microcontrollers to the uM-FPU floating point coprocessor.  It implements
; the SPI interface in software (bit-bang), and assumes a clock speed of 4 MHz.
; (There is a 20 MHz version of this file).

; The list and #include directives should be changed if another PIC device is
; used that has compatible features.
;
; The pin assignments for FPU_CLOCK, FPU_DATAIN and FPU_DATAOUT can be changed
; as required.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V2.0 November 21, 2004
; changes:
;	V2.0 November 21, 2004
;	  - updated for uM-FPU V2.0
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list	p=16f877					;list directive to define processor
	#include <p16f877.inc>				;processor specific definitions
	
	extern	delay_ms, delay

;-------------------- variable definitions ------------------------------------
		udata
dataBits		res	1					;data bits
bitCount		res	1					;bit count

#define	FPU_CLOCK		PORTC, 3		;clock (uM-FPU SCLK)
#define FPU_CLOCK_DIR	TRISC, 3		;direction bit
#define	FPU_DATAIN		PORTC, 4		;data input(uM-FPU SOUT)
#define FPU_DATAIN_DIR	TRISC, 4		;direction bit
#define	FPU_DATAOUT		PORTC, 5		;data output (uM-FPU SIN)
#define FPU_DATAOUT_DIR	TRISC, 5		;direction bit

#define WRITEBYTE_DELAY	0				;additonal delay for write data period
										;(to ensure write data period of 15 usec)
#define READBYTE_DELAY	0				;additional delay for read byte delay
										;(to ensure read byte delay of 15 usec)
#define READ_DELAY		.180			;read setup delay (180 usec)

#define FSYNC		0xF0				;synchronization opcode
#define SYNC_CHAR	0x5C				;sync character

PROG1	code

;-------------------- fpu_reset -----------------------------------------------
; Resets the uM-FPU.
;	input:	none
;	output:	Z flag = 1, Reset successful
;			Z flag = 0, Reset failed 
;------------------------------------------------------------------------------

		global		fpu_reset
fpu_reset
	banksel		TRISC
	bcf			FPU_CLOCK_DIR			;configure FPU_CLOCK as output
	bsf			FPU_DATAIN_DIR			;configure FPU_DATAIN as input
	bcf			FPU_DATAOUT_DIR			;configure FPU_DATAOUT as output
	banksel		PORTC

	bcf			FPU_CLOCK				;reset the uM-FPU
	bcf			FPU_DATAOUT
	bsf			FPU_CLOCK
	movlw		.100					;clock high for 500 usec (100 x 5 usec)
	call		delay
	bcf			FPU_CLOCK
	movlw		.8						;wait for reset to finish
	call		delay_ms

	movlw		FSYNC					;confirm communications
	call		fpu_sendByte
	call		fpu_readDelay
	call		fpu_readByte
	sublw		SYNC_CHAR
	return

;-------------------- fpu_wait ------------------------------------------------
; Waits for ready condition.
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_wait
fpu_wait	
	btfsc		FPU_DATAIN				;wait for ready condition
	goto		fpu_wait
	return

;-------------------- fpu_sendByte --------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_sendByte
fpu_sendByte
	movwf		dataBits				;save the byte to send
	movlw		.8						;get number of bits to send
	movwf		bitCount

fpu_sendByte2
	bcf			FPU_DATAOUT				;set data output LOW
	rlf			dataBits, f				;get next data bit
	btfsc		STATUS, C				;if next bit is 1, set data output HIGH
	bsf			FPU_DATAOUT

	bsf			FPU_CLOCK				;pulse the clock
	bcf			FPU_CLOCK

	decfsz		bitCount, f				;loop for all bits
	goto		fpu_sendByte2

#if	WRITEBYTE_DELAY != 0
	movlw		WRITEBYTE_DELAY			;additional delay for write data period
	call		delay	
#endif
	return

;-------------------- fpu_readByte --------------------------------------------
; Gets 8-bit byte from uM-FPU.
;	input:	none
;	output:	W register	- 8-bit value received from uM-FPU
;			dataBits	- 8-bit value received from uM-FPU
;			Z flag 		- set according to value of W register
;------------------------------------------------------------------------------

	global		fpu_readByte
fpu_readByte
	movlw		.8						;get number of bits to receive
	movwf		bitCount

fpu_readByte2
	bcf			STATUS, C				;set carry to zero
	btfsc		FPU_DATAIN				;check data input
	bsf			STATUS, C				;if HIGH, set carry to one
	bsf			FPU_CLOCK				;set clock HIGH
	rlf			dataBits, f				;store next data bit
	bcf			FPU_CLOCK				;set clock LOW
	decfsz		bitCount, f				;loop for all bits
	goto		fpu_readByte2

#if	READBYTE_DELAY != 0
	movlw		READBYTE_DELAY			;additional delay for read data delay
	call		delay	
#else
	nop
#endif

	movf		dataBits, w				;get the return byte
	return

;-------------------- fpu_readDelay -------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_readDelay
fpu_readDelay
	movlw		READ_DELAY				;delay before read
	call		delay	
	return

	end

